#------------------------------------------------------------------------------
#
#   Interesting Times - Runner
#
#------------------------------------------------------------------------------

import os
from numpy import minimum, maximum
from pygame import Color
from albow.references import Ref
from albow.controls import ValueDisplay, Button, Label
from albow.fields import IntField
from albow.multichoice import TextMultichoice
from albow.palette_view import PaletteView
from albow.layout import Row, Column, Grid, Frame
from albow.resource import get_font, get_image
from albow.image_array import get_image_array
from albow.utils import blit_in_rect
from humerus.main_commands import MainCommandScreen
from city_view import CityView
from game import game
from runner import vector_to_route_marker
import constants as k

ref_game = Ref(game)
ref_level = ref_game.playing_level
ref_team = ref_level.team(k.BLUE_TEAM)

sel_bg_color = Color("skyblue")

#------------------------------------------------------------------------------

class LevelPlayView(CityView):

	runner_start_image = get_image("runner-start-blue.png", border = 2)
	route_marker_images = get_image_array("route-markers-blue.png", 4, border = 2)

	level = property(lambda self: game.playing_level)
	
	route_edit_cell = None
	
	def draw(self, surf):
		CityView.draw(self, surf)
		selected_actor = self.level.selected_actor
		for team in self.level.teams:
			for actor in team.actors:
				actor.draw(surf, actor is selected_actor)
		for item in self.level.news_items:
			item.draw(surf)
		if selected_actor and selected_actor.is_runner:
			self.draw_start_marker(surf, selected_actor)
			self.draw_route_markers(surf, selected_actor)
	
	def mouse_down(self, e):
		p = e.local
		actor = self.find_actor(self.level.team(k.BLUE_TEAM).actors, p)
		if actor:
			self.select_actor(actor)
		else:
			actor = self.level.selected_actor
			if actor and actor.is_runner:
				self.start_route_edit(p)
				if e.shift:
					self.route_delete()
			else:
				self.set_goal_at(p)
	
	def mouse_drag(self, e):
		self.route_edit_drag(e.local)
	
	def mouse_up(self, e):
		self.end_route_edit()
	
	def select_actor(self, actor):
		self.level.selected_actor = actor
	
	def find_actor(self, actors, p):
		#print "LevelPlayView: finding actor at", p ###
		for actor in actors:
			if actor.on_duty:
				r = actor.get_hit_rect()
				#print "LevelPlayView:", actor, "has hit rect", r ###
				if r.collidepoint(p):
					#print "LevelPlayView: hit actor", actor ###
					return actor
	
	def set_goal_at(self, p):
		#print "LevelPlayView.set_goal_at:", p ###
		actor = self.level.selected_actor
		if actor:
			#print "...actor =", actor ###
			cell = self.point_to_cell(p)
			actor.set_goal(cell)
	
	def start_route_edit(self, p):
		city = self.tile_array
		cell = city.point_to_cell(p)
		if city.tile_at(cell).is_road:
			self.route_edit_cell = cell
	
	def route_edit_drag(self, p):
		cell1 = self.route_edit_cell
		actor = self.level.selected_actor
		if cell1 is not None and actor and actor.is_runner:
			city = self.tile_array
			cell2 = city.point_to_cell(p)
			d = cell2 - cell1
			d = minimum(maximum([-1, -1], d), [1, 1])
			if d[0] or d[1]:
				marker = vector_to_route_marker.get(d)
				if marker is not None:
					actor.set_route_marker(cell1, marker)
	
	def end_route_edit(self):
		self.route_edit_cell = None
	
	def route_delete(self):
		cell = self.route_edit_cell
		actor = self.level.selected_actor
		if cell is not None and actor and actor.is_runner:
			actor.set_route_marker(cell, None)
			self.end_route_edit()
	
	def draw_start_marker(self, surf, actor):
		image = self.runner_start_image
		r = image.get_rect()
		r.center = actor.initial_position
		surf.blit(image, r)
	
	def draw_route_markers(self, surf, actor):
		city = self.tile_array
		off = k.CELL_SIZE // 2
		markers = actor.route_markers
		images = self.route_marker_images
		r = images[0].get_rect()
		for cell in markers:
			i = markers[cell]
			p = city.cell_to_point(cell) + off
			r.center = p
			surf.blit(images[i], r)
	
#------------------------------------------------------------------------------

class Display(ValueDisplay):
	pass
	#margin = 4
	#bg_color = sel_bg_color
	#font = get_font(12, "Vera.ttf")
	

class DayDisplay(Display):

	ref = ref_level.time
	align = 'r'

	def get_value(self):
		return Display.get_value(self)[0]


class TimeDisplay(Display):

	ref = ref_level.time
	#format = "Day %d   %02d:%02d"
	format = "%02d:%02d"
	#align = 'r'

	def get_value(self):
		return Display.get_value(self)[1:]


class NewsDisplay(Display):

	#align = 'r'
	format = "%4d"

	def __init__(self, **kwds):
		ValueDisplay.__init__(self, width = 100, **kwds)
	
#	def __init__(self, title, **kwds):
#		ValueDisplay.__init__(self, width = 200, **kwds)
#		self.format = title + " News: %4d"

#	def get_value(self):
#		#print "NewsDisplay: ref_team -->", ref_team.get() ###
#		return Display.get_value(self)

class CashDisplay(Display):

	ref = ref_team.cash
	#format = "Cash: %6d"
	format = "$%d"
	align = 'r'

class DebugButton(Button):

	font = get_font(9, "Vera.ttf")

#------------------------------------------------------------------------------

class GameSpeedControl(TextMultichoice):

	ref = ref_level.game_speed
	tab_stop = False
	#font = get_font(12, "Vera.ttf")
	margin = 0
	border_width = 2
	highlight_style = 'fill'
	sel_color = sel_bg_color

	def __init__(self):
		TextMultichoice.__init__(self, k.game_speeds, k.game_speed_labels)

#------------------------------------------------------------------------------

class StaffView(PaletteView):
	
	sel_color = sel_bg_color
	highlight_style = 'fill'

	level = property(lambda self: game.playing_level)
	
	list = property(lambda self: self.list_ref.get())
	
	selection = property(
		lambda self: self.level.selected_actor,
		lambda self, v: setattr(self.level, 'selected_actor', v)
	)

	def __init__(self, list_ref, sel_ref, image, **kwds):
		self.list_ref = list_ref
		self.sel_ref = sel_ref
		self.image = image
		PaletteView.__init__(self,
			cell_size = (40, image.get_height()),
			nrows = 1, ncols = 5,
			**kwds)
	
	def num_items(self):
		return len(self.list)
	
	def item_is_selected(self, i):
		return self.list[i] == self.selection
	
	def click_item(self, i, e):
		self.selection = self.list[i]
	
	def draw_item(self, surf, i, rect):
		blit_in_rect(surf, self.image, rect, 'c')
		#surf.blit(self.image, rect)

#------------------------------------------------------------------------------

class StaffButton(Button):

	font = get_font(10, "Vera.ttf")

#------------------------------------------------------------------------------

class ITPlayScreen(MainCommandScreen):

	level = property(lambda self: game.playing_level)
	blue_team = property(lambda self: self.level.team(k.BLUE_TEAM))

	def __init__(self, shell):
		MainCommandScreen.__init__(self, shell)
		self.level_view = LevelPlayView(size = self.size)
		controls = self.create_controls()
		contents = Column([self.level_view, controls], align = 'l',
			height = self.height, expand = self.level_view, spacing = 0)
		self.add(contents)
		if os.getenv("IT_DEBUG"):
			debugging = self.create_debugging_controls()
			debugging.bottomleft = (0, self.bottom)
			self.add(debugging)
	
	def create_controls(self):
		clock = Row([Label("Day"), DayDisplay(width = 20), TimeDisplay(width = 50)])
		cash = Row([Label("Cash"), CashDisplay(width = 63)])
		copies_field = IntField(ref = ref_team.copies_to_print, width = 50)
		news = Frame(Grid([
			[None, Label("Column-Inches"), Label("Copies")],
			[Label("Tomorrow's Edition"), NewsDisplay(ref = ref_team.news_collected), copies_field],
			[Label("Today's Edition"), NewsDisplay(ref = ref_team.news_published), ValueDisplay(ref = ref_team.copies, width = 40)],
		], row_spacing = 4, column_spacing = 18), margin = 4)
		speed = GameSpeedControl()
		reporters = StaffView(ref_team.reporters, ref_team.selected_reporter,
			image = get_image("reporter-blue.png"))
		runners = StaffView(ref_team.runners, ref_team.selected_runner,
			image = get_image("runner-blue.png"))
		staff_display = Column([reporters, runners], align = 'l')
		staff_buttons = Column([
			StaffButton("Hire Reporter", action = self.hire_reporter, enable = self.can_hire_reporter, width = 100),
			StaffButton("Hire Runner", action = self.hire_runner, enable = self.can_hire_runner, width = 100),
			StaffButton("Dismiss Staff", action = self.dismiss_staff, enable = self.enable_dismiss_staff, width = 100),
		])
		#staff = Row([staff_display, staff_buttons])
		result = Row([
			Column([clock, cash], align = 'l', border_width = 1, margin = 4),
			news,
			Column([speed, staff_display], align = 'l'),
			staff_buttons,
		], align = 't', width = self.width, margin = 4, spacing = 8)
		result.bg_color = Color("bisque")
		#print "play_screen: controls height =", result.height ###
		return result
	
	def create_debugging_controls(self):
		return Row([
			DebugButton("+10 News", action = self.add_news_action(10)),
			DebugButton("+100 Cash", action = self.add_cash_action(100)),
			DebugButton("3:00am", action = self.time_action(2, 59)),
			DebugButton("6:00am", action = self.time_action(5, 59)),
			DebugButton("9:00am", action = self.time_action(8, 59)),
			DebugButton("5:00pm", action = self.time_action(16, 59)),
			DebugButton("New Day", action = self.time_action(23, 59)),
		])

	def screen_runs_game(self):
		print "ITPlayScreen.screen_runs_game" ###
		return True

	def add_news_action(self, n):
		return lambda: self.blue_team.collect_news(n)

	def add_cash_action(self, n):
		def action():
			self.blue_team.cash += n
		return action

	def time_action(self, h, m):
		return lambda: self.level.set_time_of_day(h, m)

	def hire_reporter(self):
		self.blue_team.hire_reporter()
	
	def can_hire_reporter(self):
		return self.blue_team.can_hire_reporter()
	
	def hire_runner(self):
		self.blue_team.hire_runner()
	
	def can_hire_runner(self):
		return self.blue_team.can_hire_runner()
	
	def dismiss_staff(self):
		self.blue_team.dismiss_staff(self.level.selected_actor)

	def enable_dismiss_staff(self):
		actor = self.level.selected_actor
		return actor and self.blue_team.can_dismiss_staff(actor)
